'use strict';

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var viewer = require('./viewer');

var BundleAnalyzerPlugin = function () {
  function BundleAnalyzerPlugin(opts) {
    (0, _classCallCheck3.default)(this, BundleAnalyzerPlugin);

    this.opts = (0, _extends3.default)({
      openAnalyzer: true,
      analyzerPort: 8888,
      generateStatsFile: false,
      statsFilename: 'stats.json'
    }, opts);
  }

  BundleAnalyzerPlugin.prototype.apply = function apply(compiler) {
    var _this = this;

    compiler.plugin('emit', function (curCompiler, callback) {
      var stats = curCompiler.getStats().toJson({
        source: false,
        warnings: false,
        errors: false,
        errorDetails: false
      });

      if (_this.opts.generateStatsFile) {
        (function () {
          var statsStr = (0, _stringify2.default)(stats, null, 2);

          curCompiler.assets[_this.opts.statsFilename] = {
            source: function source() {
              return statsStr;
            },
            size: function size() {
              return statsStr.length;
            }
          };
        })();
      }

      if (_this.opts.openAnalyzer) {
        // Making analyzer logs to be after all webpack warnings in the console
        setTimeout(function () {
          console.log('');

          viewer.start(stats, {
            port: _this.opts.analyzerPort,
            bundleDir: compiler.outputPath
          });
        }, 500);
      }

      callback();
    });
  };

  return BundleAnalyzerPlugin;
}();

module.exports = BundleAnalyzerPlugin;