#! /usr/bin/env node
'use strict';

var path = require('path');

var commander = require('commander');

var _require = require('chalk');

var bold = _require.bold;
var magenta = _require.magenta;


var analyzer = require('../analyzer');
var viewer = require('../viewer');

var program = commander.version(require('../../package.json').version).usage('<bundleStatsFile> [bundleDir] [options]\n\n  Arguments:\n  \n    bundleStatsFile  Path to Webpack Stats JSON file.\n                     It can be generated with ' + bold('webpack --profile --json > stats.json') + '\n    bundleDir        Directory containing all generated bundles. By default a directory of stats file is used.').option('-p, --port <n>', 'Port that will be used by bundle analyzer to start HTTP server.', Number, 8888).parse(process.argv);

var port = program.port;
var _program$args = program.args;
var bundleStatsFile = _program$args[0];
var bundleDir = _program$args[1];


if (!bundleStatsFile) showHelp('Provide path to Webpack Stats file as first argument');
if (isNaN(port)) showHelp('Invalid port number');

if (!bundleDir) bundleDir = path.dirname(bundleStatsFile);

var bundleStats = void 0;
try {
  bundleStats = analyzer.readStatsFromFile(bundleStatsFile);
} catch (err) {
  console.error('Could\'t read webpack bundle stats from "' + bundleStatsFile + '":\n' + err);
  process.exit(1);
}

viewer.start(bundleStats, { bundleDir: bundleDir, port: port });

function showHelp(error) {
  if (error) console.log('\n  ' + magenta(error));
  program.outputHelp();
  process.exit(1);
}