'use strict';

var fs = require('fs');
var _ = require('lodash');
var acorn = require('acorn');
var walk = require('acorn/dist/walk');

module.exports = {
  getModuleSizesFromBundle: getModuleSizesFromBundle
};

function getModuleSizesFromBundle(bundlePath) {
  var ast = acorn.parse(fs.readFileSync(bundlePath, 'utf8'), {
    sourceType: 'script'
  });

  var walkState = {
    sizes: null
  };

  walk.recursive(ast, walkState, {
    CallExpression: function CallExpression(node, state) {
      if (state.sizes) return;

      var args = node.arguments;

      if (_.get(node, 'callee.name') === 'webpackJsonp') {
        // Additional bundle without webpack loader
        // Modules are stored in second argument
        state.sizes = getModulesSizesFromFunctionArgument(args[1]);
        return;
      }

      if (args.length === 1 && (args[0].type === 'ObjectExpression' || args[0].type === 'ArrayExpression')) {
        // Main bundle with webpack loader
        // Modules are stored in first argument
        state.sizes = getModulesSizesFromFunctionArgument(args[0]);
      }
    }
  });

  return walkState.sizes;
}

function getModulesSizesFromFunctionArgument(arg) {
  if (arg.type === 'ObjectExpression') {
    var modulesNodes = arg.properties;

    return _.transform(modulesNodes, function (result, moduleNode) {
      var moduleId = moduleNode.key.name || moduleNode.key.value;
      var moduleBody = moduleNode.value.body;

      result[moduleId] = moduleBody.end - moduleBody.start;
    }, {});
  } else {
    var _modulesNodes = arg.elements;

    return _.transform(_modulesNodes, function (result, moduleNode, i) {
      if (!moduleNode) return;

      result[i] = moduleNode.body.end - moduleNode.body.start;
    }, {});
  }
}