'use strict';

var _create = require('babel-runtime/core-js/object/create');

var _create2 = _interopRequireDefault(_create);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _ = require('lodash');
var filesize = require('filesize');

var Node = function () {
  function Node(name, parent) {
    (0, _classCallCheck3.default)(this, Node);

    this.name = name;
    this.parent = parent;
  }

  Node.prototype.toString = function toString(indent) {
    indent = indent || '|';

    return indent + ' ' + this.name;
  };

  (0, _createClass3.default)(Node, [{
    key: 'path',
    get: function get() {
      var path = [];
      var node = this;

      while (node) {
        path.push(node.name);
        node = node.parent;
      }

      return path.reverse().join('/');
    }
  }]);
  return Node;
}();

var File = function (_Node) {
  (0, _inherits3.default)(File, _Node);

  function File(name, data, parent) {
    (0, _classCallCheck3.default)(this, File);

    var _this = (0, _possibleConstructorReturn3.default)(this, _Node.call(this, name, parent));

    _this.data = data;
    return _this;
  }

  File.prototype.toString = function toString(indent) {
    return _Node.prototype.toString.call(this, indent) + ' [' + this.data.id + '] (' + filesize(this.size) + ' / ' + filesize(this.parsedSize) + ')';
  };

  File.prototype.toChartData = function toChartData() {
    return {
      id: this.data.id,
      label: this.name,
      path: this.path,
      weight: this.parsedSize === undefined ? this.size : this.parsedSize,
      statSize: this.size,
      parsedSize: this.parsedSize
    };
  };

  (0, _createClass3.default)(File, [{
    key: 'size',
    get: function get() {
      return this.data.size;
    }
  }, {
    key: 'parsedSize',
    get: function get() {
      return this.data.parsedSize;
    }
  }]);
  return File;
}(Node);

var Folder = function (_Node2) {
  (0, _inherits3.default)(Folder, _Node2);

  function Folder(name, parent) {
    (0, _classCallCheck3.default)(this, Folder);

    var _this2 = (0, _possibleConstructorReturn3.default)(this, _Node2.call(this, name, parent));

    _this2.children = (0, _create2.default)(null);
    return _this2;
  }

  Folder.prototype.getChild = function getChild(name) {
    return this.children[name];
  };

  Folder.prototype.addFolder = function addFolder(name) {
    var folder = new Folder(name, this);

    this.children[name] = folder;
    delete this._size;
    delete this._parsedSize;

    return folder;
  };

  Folder.prototype.addFile = function addFile(name, data) {
    var overwrite = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

    var file = new File(name, data, this);

    if (this.children[name] && !overwrite) {
      return this.children[name];
    }

    this.children[name] = file;
    delete this._size;
    delete this._parsedSize;

    return file;
  };

  Folder.prototype.addFileByPath = function addFileByPath(path, data) {
    var _ref = [path.slice(0, -1), _.last(path)];
    var folderNames = _ref[0];
    var fileName = _ref[1];

    var currentFolder = this;

    _.each(folderNames, function (folderName) {
      currentFolder = currentFolder.getChild(folderName) || currentFolder.addFolder(folderName);
    });

    currentFolder.addFile(fileName, data);
  };

  Folder.prototype.walk = function walk(walker) {
    var state = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    var stopped = false;

    _.each(this.children, function (child) {
      if (child.walk) {
        state = child.walk(walker, state, stop);
      } else {
        state = walker(child, state, stop);
      }

      if (stopped) return false;
    });

    return state;

    function stop(finalState) {
      stopped = true;
      return finalState;
    }
  };

  Folder.prototype.toString = function toString(indent, opts) {
    var _ref2 = opts || {};

    var sortBy = _ref2.sortBy;

    indent = indent || '|';

    var str = indent + ' ' + this.name + ' (' + filesize(this.size) + ' / ' + filesize(this.parsedSize) + ')\n';

    str += _(this.children).sortBy(sortBy).reverse().map(function (child) {
      return child.toString(indent + '  |', opts);
    }).join('\n');

    return str;
  };

  Folder.prototype.toChartData = function toChartData() {
    return {
      label: this.name,
      path: this.path,
      weight: this.parsedSize === undefined ? this.size : this.parsedSize,
      statSize: this.size,
      parsedSize: this.parsedSize,
      groups: _.invokeMap(this.children, 'toChartData')
    };
  };

  (0, _createClass3.default)(Folder, [{
    key: 'size',
    get: function get() {
      if (!_.has(this, '_size')) {
        this._size = this.walk(function (node, size) {
          return size + node.size;
        }, 0);
      }

      return this._size;
    }
  }, {
    key: 'parsedSize',
    get: function get() {
      if (!_.has(this, '_parsedSize')) {
        this._parsedSize = this.walk(function (node, size, stop) {
          if (node.parsedSize === undefined) {
            return stop(undefined);
          }

          return size + node.parsedSize;
        }, 0);
      }

      /*if (this.name === 'bluebird') {
        console.log(this._parsedSize);
      }*/

      return this._parsedSize;
    }
  }]);
  return Folder;
}(Node);

module.exports = {
  Node: Node,
  File: File,
  Folder: Folder
};